package com.topsunit.scanservice.ximai.service;

import cn.hutool.core.bean.BeanUtil;
import cn.hutool.core.util.StrUtil;
import com.topsunit.scanservice.ximai.common.DateUtil;
import com.topsunit.scanservice.ximai.common.TopsunitException;
import com.topsunit.scanservice.ximai.dao.CoptgDao;
import com.topsunit.scanservice.ximai.dao.CopthDao;
import com.topsunit.scanservice.ximai.dao.CoptnDao;
import com.topsunit.scanservice.ximai.dao.CoptoDao;
import com.topsunit.scanservice.ximai.dto.*;
import com.topsunit.scanservice.ximai.entity.*;
import com.topsunit.scanservice.ximai.utils.MessageUtils;
import org.apache.commons.lang3.StringUtils;
import org.hibernate.service.spi.ServiceException;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;

import javax.transaction.Transactional;
import java.math.BigDecimal;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Optional;

@Service
public class CopthService {

    @Autowired
    private CopthDao copthDao;
    @Autowired
    private CoptgDao coptgDao;
    @Autowired
    private CoptnDao coptnDao;
    @Autowired
    private CoptoDao coptoDao;

    @Transactional
    public void create(StockSaleOutParams params){
        Coptn coptn = coptnDao.findFirstByTn001AndTn002("251", params.getApplyNo())
                .orElseThrow(()->new TopsunitException(MessageUtils.getMessage("未找到出货通知单：")+params.getApplyNo()));
        BigDecimal total = BigDecimal.ZERO;
        Calendar curr = Calendar.getInstance();
        String currStr = cn.hutool.core.date.DateUtil.format(curr.getTime(), "yyyyMMdd");
        Coptg coptg = BeanUtil.copyProperties(new CoptgCreateParams(), Coptg.class);
        coptg.setTg002(this.getNewTg002(coptg.getTg001()));
        coptg.setTg003(currStr);
        coptg.setTg004(coptn.getTn004());
        coptg.setTg008(coptn.getTn009());
        coptg.setTg009(coptn.getTn010());
        coptg.setTg011(coptn.getTn013());
        coptg.setTg012(coptn.getTn014());
        coptg.setTg044(coptn.getTn030());
        int i = 1;
        for(StockSaleOutParams.StockSaleOutDetail s : params.getData()){
            s.setWarehouse(StrUtil.trim(s.getWarehouse()));
            s.setLocation(StrUtil.trim(s.getLocation()));
            //查询出货通知单明细
            Copto copto = coptoDao.findFirstByTo001AndTo002AndTo007("251", params.getApplyNo(), s.getMaterialNo())
                    .orElseThrow(()->new TopsunitException(MessageUtils.getMessage("未找到出货通知单明细:{0},{1}", params.getApplyNo(), s.getMaterialNo())));
            CopthCreateParams copthCreateParams = new CopthCreateParams();
            Copth copth = BeanUtil.copyProperties(copthCreateParams, Copth.class);
            copth.setTh001(coptg.getTg001());
            copth.setTh002(coptg.getTg002());
            copth.setTh003(StringUtils.leftPad(i+"",4, "0"));
            copth.setTh004(s.getMaterialNo());
            copth.setTh005(s.getMaterialName());
            copth.setTh007(s.getWarehouse());
            copth.setTh008(s.getOutQuantity());
            copth.setTh009(s.getUnit());
            copth.setTh006(copto.getTo009());
            copth.setTh014(copto.getTo004());
            copth.setTh015(copto.getTo005());
            copth.setTh016(copto.getTo006());
            copth.setTh045(copto.getTo001());
            copth.setTh046(copto.getTo002());
            copth.setTh047(copto.getTo003());
            copthDao.save(copth);
            total = total.add(s.getOutQuantity());
            i++;
        }
        coptg.setTg033(total);
        coptgDao.save(coptg);
    }

    private String getNewTg002(String tg001) {
        String prefix = DateUtil.currentDateString();
        return coptgDao.findFirstByTg001AndTg002StartingWithOrderByTg002Desc(tg001, prefix)
                .map(i -> {
                    int currentOrdianl = Integer.parseInt(i.getTg002().replace(prefix, ""));
                    return prefix + String.format("%03d", currentOrdianl + 1);
                })
                .orElse(prefix + "001");
    }

}
